import {html, css, unsafeCSS} from 'lit-element';
import moment from 'moment';
import flatpickr from 'flatpickr';
import styles from 'flatpickr/dist/themes/light.css';
import BaseComponent from './BaseComponent.js';

class CWDateBox extends BaseComponent {
  static get styles() {
    return [unsafeCSS(styles), css`
      :host {
        display: block;
        padding: 25px;
        color: var(--x-test-widget-text-color, #000);
      }
    `];
  }

  static get properties() {
    return {
      options: {type: Object},
      label: {type: String},
      description: {type: String},
      access: {type: String},
      value: {type: String},
      excludedDates: {type: Array},
      minDate: {type: String},
      maxDate: {type: String}
    };
  }

  constructor() {
    super();
    this.format = 'd/m/Y'; // TODO: should be got from l10n? also see https://flatpickr.js.org/formatting/, maybe shoud convert typing defenitions?
    this.internalFormat = 'YYYY-MM-DD';
    this.submittable = true;

    this.valid = true;
    this.excludedDates = ['2020-12-07'];
  }

  firstUpdated() {
    super.firstUpdated();
    this.flatpickr = flatpickr(this.shadowRoot.querySelector('.datepicker'), {
      appendTo: this.shadowRoot,
      allowInput: true,
      dateFormat: this.format,
      onChange: (selectedDates, dateStr) => {
        // TODO: call this.setValue
        this.value = selectedDates;
      },
    });
  }

  render() {
    return html`
      <label>${this.label}</label>
      <input class="datepicker" />
      <div class="description">${this.description}</div>
      ${this.valid === true ? '' : html`<div class="error">${this.valid}</div>`}
    `;
  }

  get options() {
    return this._options;
  }

  set options(opts) {
    const options = {};
    Object.keys(opts).forEach(key => {
      switch (key) {
        case 'excludedDates': {
          if (typeof opts[key] === 'string') {
            options[key] = opts[key].split(/\s*,\s*/);
          }
          break;
        }
        case 'minDate': {
          try {
            options[key] = opts[key] != null ? moment(opts[key]).format(this.internalFormat) : null;
          } catch (e) {
            // TODO: log error
            // console.debug("%c%s: %cIncorrect minDate: %s", "color: dimgrey", widget.element.attr("id"), "color: red", value);
            options[key] = null;
          }

          break;
        }
        case 'maxDate': {
          try {
            options[key] = opts[key] != null ? moment(opts[key]).format(this.internalFormat) : null;
          } catch (e) {
            // TODO: log error
            // console.debug("%c%s: %cIncorrect minDate: %s", "color: dimgrey", widget.element.attr("id"), "color: red", value);
            options[key] = null;
          }
          break;
        }
        default:
      }
    });
    const oldVal = this._options;
    this._options = opts;
    // TODO: mask?
    // this.maskOptions = {...this.maskOptions, min: opts.minDate, max: opts.maxDate};
    this.requestUpdate('options', oldVal);
  }


  setValue(value, context) {
    // TODO: refactor it somehow please, it should be better than original =) do we need two different events or context?
    // TODO: also, move base logic to parent component class or mixin ('withValue' or something else) maybe?
    value = typeof value === 'string' ? moment(value).toDate() : value;
    if (value == null && this.value == null || value != null && moment(value).isSame(this.value)) {
      return Promise.resolve();
    }
    const prevValue = this.value;
    this.value = value;
    // TODO: mask?
    // this.maskedInput.value = value != null ? moment(this.value).format(this.format) : value;
    this.flatpickr.setDate(value != null ? moment(this.value).format(this.format) : value);

    if (context === 'fromUI') {
      if (!this.changed) {
        this.changed = true;
        this.validate('silent');
      } else {
        this.validate();
      }
      this.addHostClass('xwidget_changed');
      this.dispatchEvent(new CustomEvent('valueChanged'));
    } else {
      this.prevValue = prevValue;
      this.dispatchEvent(new CustomEvent('valuePopulated', {detail: context}));
    }

    return Promise.resolve();
  }

  getValue(context) {
    // TODO: refactor it somehow please, it should be better than original =)
    if (this.value == null) {
      return null;
    }
    if (context) {
      return moment(this.value).format(this.internalFormat);
    } else {
      return this.value;
    }
  }

  // TODO: add defaultValue and setDefaultValue method in bridge maybe?

  clear() {
    this.value = null;
    this.flatpickr.clear();
  }

  validate() {
    // TODO: refactor it somehow please
    let valid = true;
    if (this.access === 'required') {
      valid = this.value != null;
    }
    if (this.value != null) {
      if (this.options.maxDate != null && moment(this.value).diff(moment(this.options.maxDate)) > 0) {
        valid = 'form.message.incorrect.max';
        // valid = l10n.get("form.message.incorrect.max", widget.option("label"), $.datepicker.formatDate(widget.option("dateFormat"), new Date(widget.options.maxDate)));
      } else if (this.options.minDate != null && moment(this.value).diff(moment(this.options.minDate)) < 0) {
        valid = 'form.message.incorrect.min';
        // valid = l10n.get("form.message.incorrect.min", widget.option("label"), $.datepicker.formatDate(widget.option("dateFormat"), new Date(widget.options.minDate)));
      }
    }
    if (valid !== true) {
      this.addHostClass('xwidget_error');
    } else {
      this.removeHostClass('xwidget_error');
    }
    this.valid = valid;
    this.requestUpdate();
    return valid;
  }

}

window.customElements.define('cw-date-box', CWDateBox);
