import {LitElement, html, css} from 'lit-element';

/**
 * A simple toggle component
 * @element x-toggle
 * @slot {title} - Label for toggle
 * @attr {Boolean} disabled - Disable toggle
 * @attr {Boolean} checked - check toggle
 * @cssprop {Color} --checkbox-color - Controls the color of the toggle
 * @fires {} change - Fires when checked value changes
 */
class XToggle extends LitElement {
    static get properties() {
        return {
            checked: {type: Boolean, attribute: true, reflect: true},
            disabled: {type: Boolean, attribute: true, reflect: true},
        };
    }

    static get styles() {
        return css`
        label {
            display: inline-flex;
            align-items: center;
        }
        span {
            position: relative;
            cursor: pointer;
            margin-rigth: 5px;
        }
        #checkbox {
            position: absolute;
            opacity: 0;
            pointer-events: none;
        }
        span::before,
        span::after {
            content: '';
            display: block;
            margin: 0 3px;
            transition: all 80ms linear;
            background: #BDBDBD;
        }
        span::before {
            height: 0.9rem;
            width: 2.4rem;
            border-radius: 0.65em;
            opacity: 0.6;
        }
        span::after {
            position: absolute;
            top: 50%;
            transform: translate(0, -50%);
            height: 1.3em;
            width: 1.3em;
            border-radius: 50%;
            box-shadow: rgba(0, 0, 0, 0.2) 0px 2px 1px -1px, rgba(0, 0, 0, 0.14) 0px 1px 1px 0px, rgba(0, 0, 0, 0.12) 0px 1px 3px 0px;
            background-color: rgb(250, 250, 250);                           
        }
        span:hover:after {
            box-shadow: rgba(0, 0, 0, 0.2) 0px 2px 1px -1px, rgba(0, 0, 0, 0.14) 0px 1px 1px 0px, rgba(0, 0, 0, 0.12) 0px 1px 3px 0px, 0 0 2px 10px rgba(158, 158, 158, 0.08);           
        }        
        // #checkbox:focus+span {
        //     box-shadow: rgba(94, 158, 214, 0.2) 0 0 8px
        // }
        #checkbox:disabled+span {
            cursor: not-allowed;
            filter: grayscale(100%);
            opacity: 0.6;
        }
        #checkbox:disabled+span::after {
            box-shadow: none;
        }
        #checkbox:checked+span::before,
        #checkbox:checked+span::after{
            background: var(--checkbox-color, #1EA7FD);
        }
        #checkbox:checked+span::after {
            transform: translate(calc(2.4em - 100%), -50%);
        }
        `;
    }

    updated(changedProperties) {
        if (changedProperties.has("checked"))
            this.dispatchEvent(new Event('change', {bubbles: true}));
    }

    updateChecked(e) {
        this.checked = e.target.checked;
    }

    render() {
        return html`
            <label for="checkbox">
                <input id="checkbox" type="checkbox" .checked="${this.checked}" ?disabled="${this.disabled}" @change="${this.updateChecked}"/>
                <span></span>
                <slot></slot>
            </label>
        `;
    }
}

window.customElements.define('x-toggle', XToggle);